//
//  JPTableViewController.m
//  Octotco
//
//  Created by Johannes Plunien on 11/17/11.
//  Copyright (c) 2011 __MyCompanyName__. All rights reserved.
//

#import "JPTableViewController.h"

@implementation JPTableViewController

@synthesize rows, isLoadingMoreRows, linkHeader;

// Obviously this must be overridden in subclasses.
- (NSString *)getObjectClass
{
    return @"NSObject";
}

- (void)loadRowsAtResourcePath:(NSString *)path
{
    RKObjectMapping* mapping = [[RKObjectManager sharedManager].mappingProvider objectMappingForClass:NSClassFromString([self getObjectClass])];
    [[RKObjectManager sharedManager] loadObjectsAtResourcePath:path objectMapping:mapping delegate:self];
    self.isLoadingMoreRows = true;
    [self.tableView reloadData];
}

#pragma mark UITableViewDataSource methods

- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section
{
    if (self.isLoadingMoreRows) {
        return [self.rows count] + 1;
    }
    return [self.rows count];
}

#pragma mark UITableViewDelegate methods

- (void)scrollViewDidScroll:(UIScrollView *)scrollView
{
    CGPoint offset = scrollView.contentOffset;
    CGRect bounds = scrollView.bounds;
    CGSize size = scrollView.contentSize;
    UIEdgeInsets inset = scrollView.contentInset;
    float y = offset.y + bounds.size.height - inset.bottom;
    float h = size.height;
    if(y >= h) {
        NSURL *url = [self.linkHeader getUrl:@"next"];
        if ([self.rows count] > 0 && url != nil && !self.isLoadingMoreRows) {
            NSString *path = [[[NSString alloc] initWithFormat:@"%@?%@", url.path, url.query] autorelease];
            [self loadRowsAtResourcePath:path];
        }
    }
}

#pragma mark - RKObjectLoaderDelegate methods

- (void)objectLoader:(RKObjectLoader *)objectLoader didLoadObjects:(NSArray *)objects
{
    NSMutableArray *result = [[[NSMutableArray alloc] initWithArray:self.rows] autorelease];
    [result addObjectsFromArray:objects];
    self.isLoadingMoreRows = false;
    self.rows = result;
    [self.tableView reloadData];
}

- (void)objectLoaderDidFinishLoading:(RKObjectLoader *)objectLoader
{
    self.linkHeader = [[[JPLinkHeader alloc] initWithString:[objectLoader.response.allHeaderFields valueForKey:@"Link"]] autorelease];
}

- (void)objectLoader:(RKObjectLoader *)objectLoader didFailWithError:(NSError *)error
{
    self.isLoadingMoreRows = false;
    [self.tableView reloadData];
    NSLog(@"Hit error: %@", error);
}

#pragma mark - Lifecycle

- (void)dealloc
{
    self.rows = nil;
    self.linkHeader = nil;
}

@end
